//================================================================
//=	  programowa obsluga magistrali I2C
//=	  CPU = ATMega8515
//=   CLK = 8MHz
//================================================================

#include <avr/io.h>
#include "vector_disp.h"
#include "i2c.h"



//================================================================
//===== Start/Stop magistrali I2C ================================
//================================================================
void I2C_Start(void)						// wczesniej I2C = 0
{
	I2C_DIR &=~ SDA;						// SDA => in => H
	delay(16);								// 6us
	I2C_DIR &=~ SCL;						// SCL => in => H
	delay(16);								// 6us
	I2C_DIR |= SDA;							// SDA => out => L
	delay(16);								// 6us
	I2C_DIR |= SCL;							// SCL => out => L
	delay(16);								// 6us
}


void I2C_Stop(void)
{
	I2C_DIR |= SDA;							// SDA => out => L
	delay(16);								// 6us
	I2C_DIR &=~ SCL;						// SCL => in => H
	delay(16);								// 6us
	I2C_DIR &=~ SDA;						// SDA => in => H
	delay(16);								// 6us
}
//================================================================



//================================================================
//===== Wysylanie/Odbior pojedynczych bajtow =====================
//================================================================
unsigned char I2C_Read(unsigned char ack)	// zwraca odebrany bajt
{
	char ii, dane = 0;
 
	for(ii=0;ii<8;ii++)
	{
		I2C_DIR |= SCL;						// SCL => out => L
		delay(16);							// 6us
		I2C_DIR &=~ SCL;					// SCL => in => H
		delay(16);							// 6us
	
		dane <<= 1;							// przesun odebrany bit
		if((I2C_IN & SDA) == 1) dane++;		// odczytaj bit
	}
	
	I2C_DIR |= SCL;							// SCL => out => L
	if(ack)
		I2C_DIR |= SDA;						// SDA => out => L 
	else
		I2C_DIR &=~ SDA;					// SDA => in => H
	
	delay(16);								// 6us
	I2C_DIR &=~ SCL;						// SCL => in => H
	while(!(I2C_IN & SCL));					// poczekaj na gotowosc slave'a
	
	delay(16);								// 6us
	I2C_DIR &=~ SDA;						// SDA => in => H
	I2C_DIR |= SCL;							// SCL => out => L
	
	return(dane);
}


unsigned char I2C_Send(unsigned char bajt)	// wyslanie bajtu
{
	unsigned char ii;

	for(ii=0;ii<8;ii++)
	{
		if(bajt & 0x80) 
			I2C_DIR &=~ SDA;				// wystaw bit = 1
		else I2C_DIR |= SDA;				// wystaw bit = 0
		delay(16);							// 6us
		
		I2C_DIR &=~ SCL;					// SCL => in => H
		delay(16);							// 6us
		I2C_DIR |= SCL;						// SCL => out => L
		delay(16);							// 6us
		
		bajt <<= 1;							// przesun wysylany bajt w lewo
	}
	
	I2C_DIR &=~ SDA;						// SDA => in => H
	delay(16);								// 6us
	I2C_DIR &=~ SCL;						// SCL => in => H
	while(!(I2C_IN & SCL));					// poczekaj na gotowosc slave'a

	delay(16);								// 6us
	ii = I2C_IN & SDA;						// odczyt ACK
	return ii;								// zwroc ACK	
}
//================================================================



//================================================================
//===== Komunikacja I2C ==========================================
//================================================================
void I2C_Wyslij(unsigned char adr_I2C, unsigned char adr_bajtu, unsigned char ile_bajtow, unsigned char* bufor)
{
	unsigned char i1=0;

	I2C_Start();							// start magistrali I2C
	I2C_Send(adr_I2C);						// wyslij adres odbiorcy (bez obslugi bledu!!!)
	I2C_Send(adr_bajtu);					// wyslij adres rejestru odbiorcy
	
	while(i1 < ile_bajtow)
	{
		I2C_Send(bufor[i1]);				// wyslij kolejny bajt
		i1++;
	}
	
	I2C_Stop();								// zwolnij magistrale I2C
}


void I2C_Odbierz(unsigned char adr_I2C, unsigned char adr_bajtu, unsigned char ile_bajtow, unsigned char* bufor)
{
	unsigned char i1=0;
	
	I2C_Start();							// start magistrali I2C
	I2C_Send(adr_I2C);						// wyslij adres odbiorcy (bez obslugi bledu!!!)	
	I2C_Send(adr_bajtu);					// wyslij adres rejestru odbiorcy
	
	I2C_Start();							// ponownie wyslij sekwencje startu
	I2C_Send(adr_I2C + 1);					// wyslij adres odbiorcy+1 (odczyt) 
	
	while(i1 < (ile_bajtow - 1))
	{
		bufor[i1] = I2C_Read(1);			// odbierz kolejny bajt
		i1++;
	}
	
	bufor[i1] = I2C_Read(0);				// odbierz ostatni bajt (bez ACK)

	I2C_Stop();								// zwolnij magistrale I2C
}
//================================================================

